{# The values of the following custom heat parameters was hard-coded into this template:
 * ControlPlaneEdge1SubnetCidr
 * ControlPlaneEdge1DefaultRoute
 * Edge1EC2MetadataIp
 * StorageSupernet
 * InternalApiSupernet
 * TenantSupernet
To parameterize use the {{role.name}}ExtraGroupVars THT interface and update the template to use an ansible var.
#}
---
network_config:
- type: interface
  name: nic1
  use_dhcp: false
  dns_servers: {{ ctlplane_dns_nameservers }}
  addresses:
  - ip_netmask: {{ ctlplane_ip }}/24{# NOTE! Custom parameter ControlPlaneEdge1SubnetCidr was hard-coded in the converted template. To parameterize use the {role.name}ExtraGroupVars THT interface and update the template to use an ansible var. #}
  routes: {{ [{'default': True, 'next_hop': '172.16.1.201'}, [{'ip_netmask': '192.168.254.0/24',
    'next_hop': '192.168.24.1'}], [{'ip_netmask': '169.254.169.254/32', 'next_hop': '172.16.1.201'}]]
    | flatten | unique }}
- type: ovs_bridge
  name: br-ex
  use_dhcp: false
  members:
  - type: interface
    name: nic2
    primary: true
  - type: vlan
    vlan_id: {{ storageedge1_vlan_id }}
    addresses:
    - ip_netmask: {{ storageedge1_ip }}/{{ storageedge1_cidr }}
    routes:
    - ip_netmask: 172.18.0.0/16{# NOTE! Custom parameter StorageSupernet was hard-coded in the converted template. To parameterize use the {role.name}ExtraGroupVars THT interface and update the template to use an ansible var. #}
      next_hop: {{ storageedge1_gateway_ip }}
  - type: vlan
    vlan_id: {{ internal_apiedge1_vlan_id }}
    addresses:
    - ip_netmask: {{ internal_apiedge1_ip }}/{{ internal_apiedge1_cidr }}
    routes:
    - ip_netmask: 172.20.0.0/16{# NOTE! Custom parameter InternalApiSupernet was hard-coded in the converted template. To parameterize use the {role.name}ExtraGroupVars THT interface and update the template to use an ansible var. #}
      next_hop: {{ internal_apiedge1_gateway_ip }}
  - type: vlan
    vlan_id: {{ tenantedge1_vlan_id }}
    addresses:
    - ip_netmask: {{ tenantedge1_ip }}/{{ tenantedge1_cidr }}
    routes:
    - ip_netmask: 172.21.0.0/16{# NOTE! Custom parameter TenantSupernet was hard-coded in the converted template. To parameterize use the {role.name}ExtraGroupVars THT interface and update the template to use an ansible var. #}
      next_hop: {{ tenantedge1_gateway_ip }}
