#!/usr/bin/env bash
set -e
echo "Running $0" >> /root/standalone-ansible-doi9u3ei/cephadm/cephadm_command.log
ANSIBLE_LOG_PATH="/root/standalone-ansible-doi9u3ei/cephadm/cephadm_command.log" ANSIBLE_HOST_KEY_CHECKING=false ansible-playbook -e ansible_python_interpreter=/usr/bin/python3 -e @/root/standalone-ansible-doi9u3ei/cephadm/cephadm-extra-vars-heat.yml -e @/root/standalone-ansible-doi9u3ei/cephadm/cephadm-extra-vars-ansible.yml -e @/root/standalone-ansible-doi9u3ei/global_vars.yaml -i /root/standalone-ansible-doi9u3ei/inventory.yaml /usr/share/ansible/tripleo-playbooks/cephadm.yml
 2>&1
