---
network_config:
- type: interface
  name: nic1
  mtu: {{ ctlplane_mtu }}
  use_dhcp: false
  dns_servers: {{ ctlplane_dns_nameservers }}
  domain: {{ dns_search_domains }}
  addresses:
  - ip_netmask: {{ ctlplane_ip }}/{{ ctlplane_subnet_cidr }}
  routes: {{ [ctlplane_host_routes] | flatten | unique }}
- type: interface
  name: nic2
  mtu: {{ storage_mtu }}
  use_dhcp: false
- type: vlan
  device: nic2
  mtu: {{ storage_mtu }}
  vlan_id: {{ storage_vlan_id }}
  addresses:
  - ip_netmask: {{ storage_ip }}/{{ storage_cidr }}
  routes: {{ [storage_host_routes] | flatten | unique }}
- type: interface
  name: nic5
  mtu: {{ storage_mgmt_mtu }}
  use_dhcp: false
- type: vlan
  device: nic5
  mtu: {{ storage_mgmt_mtu }}
  vlan_id: {{ storage_mgmt_vlan_id }}
  addresses:
  - ip_netmask: {{ storage_mgmt_ip }}/{{ storage_mgmt_cidr }}
  routes: {{ [storage_mgmt_host_routes] | flatten | unique }}
- type: interface
  name: nic8
  mtu: {{ internal_api_mtu }}
  use_dhcp: false
- type: vlan
  device: nic8
  mtu: {{ internal_api_mtu }}
  vlan_id: {{ internal_api_vlan_id }}
  addresses:
  - ip_netmask: {{ internal_api_ip }}/{{ internal_api_cidr }}
  routes: {{ [internal_api_host_routes] | flatten | unique }}
- type: ovs_bridge
  name: br-tenant
  mtu: {{ tenant_mtu }}
  dns_servers: {{ ctlplane_dns_nameservers }}
  use_dhcp: false
  members:
  - type: interface
    name: nic11
    mtu: {{ tenant_mtu }}
    use_dhcp: false
    primary: true
  - type: vlan
    mtu: {{ tenant_mtu }}
    vlan_id: {{ tenant_vlan_id }}
    addresses:
    - ip_netmask: {{ tenant_ip }}/{{ tenant_cidr }}
    routes: {{ [tenant_host_routes] | flatten | unique }}
- type: ovs_bridge
  name: bridge_name
  mtu: {{ external_mtu }}
  dns_servers: {{ ctlplane_dns_nameservers }}
  use_dhcp: false
  members:
  - type: interface
    name: nic14
    mtu: {{ external_mtu }}
    use_dhcp: false
    primary: true
  - type: vlan
    mtu: {{ external_mtu }}
    vlan_id: {{ external_vlan_id }}
    addresses:
    - ip_netmask: {{ external_ip }}/{{ external_cidr }}
    routes: {{ [external_host_routes, [{'default': True, 'next_hop': external_gateway_ip}]] | flatten | unique }}
